/*
 * Copyright (c) 1998-2007 TeamDev Ltd. All Rights Reserved.
 * Use is subject to license terms.
 */
function q__initDropDown(dropDownId,
                         initialText,
                         containerClass,
                         rolloverContainerClass,
                         fieldClass,
                         rolloverFieldClass,
                         buttonClass,
                         rolloverButtonClass,
                         pressedButtonClass,
                         popupClass,
                         rolloverPopupClass) {

  var dropDown = q__getControl(dropDownId);

  dropDown._field = q__getControl(dropDownId + "::field");
  dropDown._button = q__getControl(dropDownId + "::button");
  dropDown._popup = q__getControl(dropDownId + "--popup");
  if (!dropDown._field) {
    // the case for SuggestionField
    q__assert(dropDown.nodeName == "INPUT", "q__initDropDownField. Unexpected dropDown.nodeName: " + dropDown.nodeName);
    dropDown._field = dropDown;
  }

  var field = dropDown._field;
  var button = dropDown._button;
  var popup = dropDown._popup;

  field._dropDown = dropDown;
  if (button)
    button._dropDown = dropDown;
  popup._dropDown = dropDown;

  dropDown._containerClass = q__dropdown_getClassName(containerClass);
  dropDown.className = dropDown._containerClass;
  dropDown._rolloverContainerClass = dropDown._containerClass + q__dropdown_getClassName(rolloverContainerClass);

  dropDown._fieldClass = q__dropdown_getClassName(fieldClass);
  if (dropDown != field)
    field.className = dropDown._fieldClass;
  if (document.compatMode == "CSS1Compat" && !q__isOpera()) // fixes the strict mode issue of assigning 100% width on text-field inside a table
    field.parentNode.style.paddingRight = q__isExplorer() ? "8px" : "4px";

  dropDown._rolloverFieldClass = dropDown._fieldClass + q__dropdown_getClassName(rolloverFieldClass);

  dropDown._buttonClass = q__dropdown_getClassName(buttonClass);
  if (button)
    button.className = dropDown._buttonClass;
  dropDown._rolloverButtonClass = dropDown._buttonClass + q__dropdown_getClassName(rolloverButtonClass);
  dropDown._pressedButtonClass = dropDown._rolloverButtonClass + q__dropdown_getClassName(pressedButtonClass);

  var mouseOverBehaviorNeeded = rolloverContainerClass || rolloverFieldClass || rolloverButtonClass || rolloverPopupClass;

  if (button) {
    button._focusable = true;
    var buttonImage = button.getElementsByTagName("img")[0];
    buttonImage._focusable = true;

    button.onmousedown = function (e) {
      if (q__isEventFromInsideOfElement(e, popup))
        return;
      button.className = dropDown._pressedButtonClass;
      dropDown._showHidePopup();
      q__hideAllPopupsExceptOne(popup);
      q__breakEvent(e);
    }
    button.ondblclick = function (e) {
      if (q__isEventFromInsideOfElement(e, popup))
        return;
      if (!q__isExplorer())
        return;
      dropDown._showHidePopup();
      q__breakEvent(e);
    }
    button.onmousemove = function (e) {
      if (q__isEventFromInsideOfElement(e, popup))
        return;
      q__breakEvent(e);
    }
    button.onmouseup = function (e) {
      if (q__isEventFromInsideOfElement(e, popup))
        return;
      button.className = dropDown._rolloverButtonClass;
    }
    button.ondragstart = function(e) {
      q__breakEvent(e);
    }
    button.onselectstart = function (e) {
      q__breakEvent(e);
    }
  }

  popup.onmousemove = function (e) {
    q__breakEvent(e);
  }

  popupClass = q__combineClassNames([popupClass, popup.className]);
  dropDown._popupClass = q__dropdown_getClassName(popupClass);
  popup.className = dropDown._popupClass;
  dropDown._rolloverPopupClass = dropDown._popupClass + " " + q__dropdown_getClassName(rolloverPopupClass);

  if (mouseOverBehaviorNeeded) {
    dropDown._dropDownMouseOver = function() {
      if (dropDown && dropDown._containerClass != dropDown._rolloverContainerClass)
        dropDown.className = dropDown._rolloverContainerClass;
      if (dropDown != field && dropDown._fieldClass != dropDown._rolloverFieldClass)
        field.className = dropDown._rolloverFieldClass;
      if (button && dropDown._buttonClass != dropDown._rolloverButtonClass)
        button.className = dropDown._rolloverButtonClass;
      if (popup && dropDown._popupClass != dropDown._rolloverPopupClass)
        popup.className = dropDown._rolloverPopupClass;
      q__repaintAreaForOpera(dropDown, true);
      q__repaintAreaForOpera(popup, true);
    }

    dropDown._dropDownMouseOut = function() {
      if (dropDown && dropDown._containerClass != dropDown._rolloverContainerClass)
        dropDown.className = dropDown._containerClass;
      if (dropDown != field && dropDown._fieldClass != dropDown._rolloverFieldClass)
        field.className = dropDown._fieldClass;
      if (button && dropDown._buttonClass != dropDown._rolloverButtonClass)
        button.className = dropDown._buttonClass;
      if (popup && dropDown._popupClass != dropDown._rolloverPopupClass)
        popup.className = dropDown._popupClass;
      q__repaintAreaForOpera(dropDown, true);
      q__repaintAreaForOpera(popup, true);
    }

    q__addMouseOverListener(dropDown, dropDown._dropDownMouseOver);
    q__addMouseOverListener(popup, dropDown._dropDownMouseOver);

    q__addMouseOutListener(dropDown, dropDown._dropDownMouseOut);
    q__addMouseOutListener(popup, dropDown._dropDownMouseOut);
  }

  //add function to change popup position if window is resized and layout is changed
  q__addEventHandler(window, "resize", function () {
    var dropDownPopup = dropDown._popup;
    // drop-down can be removed from the page using Ajax, so we need to check its presense
    if (dropDownPopup && dropDownPopup.isVisible()) {
      q__dropDown_alignPopup(dropDown);
    }
  });

  dropDown._initValue = function (value) {
    field.value = value;
  }

  if (dropDown != field)
    dropDown.focus = function() {
      try {
        field.focus();
      }
      catch(e) {
      }
    }

  setTimeout(function() {
    dropDown._initValue(initialText);
  }, 100);
}

function q__dropdown_getClassName(param) {
  return (param == null) ? '' : ' ' + param;
}


function q__dropDown_initPopup(dropDown, calendar) {
  var popup = dropDown._popup;

  var sizeChanged = false;
  if (calendar) {
    popup.style.width = calendar.offsetWidth + "px";
  } else {
    var innerTable = document.getElementById(popup.id + "::innerTable");
    var tableWidth = innerTable.offsetWidth;
    var tableHeight = innerTable.offsetHeight;
    if (popup._initializedTableWidth != tableWidth || popup._initializedTableHeight != tableHeight) {
      popup._initializedTableWidth = tableWidth;
      popup._initializedTableHeight = tableHeight;
      sizeChanged = true;

      var minWidth;
      if (popup._initialClientWidth == undefined) {
        popup._initialClientWidth = popup.clientWidth;
      }
      if (popup.clientWidth <= 0) { // the case when width is not specified explicitly
        minWidth = dropDown.offsetWidth;
        if (tableWidth > dropDown.offsetWidth) {
          minWidth = tableWidth;
        }
      } else
        minWidth = popup.offsetWidth;

      if (popup.offsetWidth < minWidth) {
        popup.style.width = minWidth + "px";
        popup._widthCorrection = popup.offsetWidth - minWidth;
        if (popup._widthCorrection) // actual for Mozilla & Opera
          popup.style.width = (minWidth - popup._widthCorrection) + "px";
      }

      if (popup.offsetWidth < minWidth) {
        popup.style.width = dropDown.offsetWidth + "px";
      }

      if (popup._initialClientHeight == undefined) {
        popup._initialClientHeight = popup.clientHeight;
      } else {
        popup.style.height = popup._initialClientHeight + "px";
      }
      function adjustHeight() {
        var contentHeight = innerTable.offsetHeight;
        if (popup.clientHeight > contentHeight) {
          var borderAccomodation = (q__isExplorer() && document.compatMode == "BackCompat") ? 10 : 0;
          var preferredHeight = borderAccomodation + (contentHeight ? contentHeight : 20);
          popup.style.height = preferredHeight + "px";
          var heightCorrection = popup.clientHeight - preferredHeight;
          if (heightCorrection)
            popup.style.height = (preferredHeight - heightCorrection - borderAccomodation) + "px";
        }
      }
      adjustHeight();

      var recalculateWidth = q__isExplorer() && document.compatMode == "CSS1Compat";
      if (popup._initialClientWidth <= 0) { // if width is not specified explicitly
        if (recalculateWidth)
          innerTable.width = "";
        var contentWidth = innerTable.offsetWidth;
        var missingWidth = contentWidth - popup.clientWidth;
        if (missingWidth > 0)
          popup.style.width = popup.offsetWidth + missingWidth + "px";
        if (recalculateWidth)
          innerTable.width = popup.clientWidth;
      } else {
        if (recalculateWidth) {
          innerTable.width = popup.clientWidth;
        }
      }

      if (document.compatMode == "CSS1Compat") {
        adjustHeight();
      }
    }
  }

  q__dropDown_alignPopup(dropDown, calendar);
  if (popup._outerIEDiv)
    popup._outerIEDiv._updateLocation();
}

function q__dropDown_alignPopup(dropDown, calendar) {
  var popup = dropDown._popup;

  var dropDownPos = q__getElementPos(dropDown, false, true);

  var left;
  if (dropDown._listAlignment == 'right' || calendar) { // to the right border
    left = dropDownPos.left - ((calendar ? calendar.clientWidth : popup.clientWidth) - dropDown.clientWidth);
  } else { // bind drop list to the left border
    left = dropDownPos.left;
  }
  var top = dropDownPos.top + dropDown.offsetHeight;

  if ((q__isOpera() || q__isSafari()) && dropDown.offsetParent.tagName.toLowerCase() != 'body') {
    var leftMargin = q__getLeftBodyMargin();
    left -= leftMargin;
    var topMargin = q__getTopBodyMargin();
    top -= topMargin;
  }

  var widthCorrection = popup._widthCorrection ? popup._widthCorrection : 0;
  var topCorrection = 0;
  var leftCorrection = 0;
  if (q__isMozilla() && !q__isExplorer()) {
    var containerClasses = dropDown._containerClass;
    var cssRulesNames = q__getArrayFromString(containerClasses, " ");
    for (var i = 0, count = cssRulesNames.length; i < count; i ++) {
      var cssRule = q__findRule('.' + cssRulesNames[i]);
      if (!cssRule)
        continue;
      var s = cssRule.style;
      if (s.marginLeft) {
        var marginLeft = parseInt(s.marginLeft);
        if (!popup._widthCorrected)
          widthCorrection += marginLeft;
        leftCorrection += marginLeft;
      }
      if (s.marginRight) {
        var marginRight = parseInt(s.marginRight);
        if (!popup._widthCorrected)
          widthCorrection += marginRight;
      }
      if (s.marginTop) {
        var marginTop = parseInt(s.marginTop);
        topCorrection = marginTop;
      }
    }
    var w = popup.offsetWidth;
    w -= widthCorrection;
    popup.style.width = w + "px";

    popup._widthCorrected = true;
  }
  popup.setTop(top - topCorrection);
  popup.setLeft(left + leftCorrection);
}
